//Starter code for practical exercise #2, CS 276 Autumn 2005.
//Written by Louis Eisenberg.
//Modified by Jason Kusuma

import java.io.*;
import java.util.*;
import java.util.regex.*;
import java.text.*;

public class MyMessageParser {

    // We use all messages available for training
    // static final int MSGS_PER_GROUP = 50;

    static final String newsgroupsDir = "/home/rashmi/cs276/PE2/work/download/data/20news-18828/";

    public static String[] newsgroups = { "alt.atheism", "comp.graphics",
            "comp.os.ms-windows.misc", "comp.sys.ibm.pc.hardware",
            "comp.sys.mac.hardware", "comp.windows.x", "misc.forsale",
            "rec.autos", "rec.motorcycles", "rec.sport.baseball",
            "rec.sport.hockey", "sci.crypt", "sci.electronics", "sci.med",
            "sci.space", "soc.religion.christian", "talk.politics.guns",
            "talk.politics.mideast", "talk.politics.misc", "talk.religion.misc" };

    public static HashSet stopWords = new HashSet();

    public static ArrayList mfs = new ArrayList();

    public static String stopFile = "english.stop";

    // whole corpus
    public static HashMap allSubject = new HashMap();

    public static HashMap allBody = new HashMap();

    public static int numDocs = 0;

    public static void loadStopWords() throws IOException,
            FileNotFoundException {
        File stops = new File(stopFile);
        BufferedReader in = new BufferedReader(new FileReader(stops));
        String line;
        while ((line = in.readLine()) != null)
            stopWords.add(doStemming(line));
    }
    public int counterNewsgroup(int newsgroup) throws IOException,
    FileNotFoundException {
        File directory = new File(newsgroupsDir + newsgroups[newsgroup]);
        if (!directory.exists()) {
            System.out.println("Can't locate directory for "
            + newsgroups[newsgroup]);
            System.exit(-1);
        }
        File[] msgs = directory.listFiles();

        return msgs.length;
    }

    public static void parseNewsgroup(int newsgroup) throws IOException,
            FileNotFoundException {
        File directory = new File(newsgroupsDir + newsgroups[newsgroup]);
        if (!directory.exists()) {
            System.out.println("Can't locate directory for "
                    + newsgroups[newsgroup]);
            System.exit(-1);
        }
        File[] msgs = directory.listFiles();
        //System.out.println("Parsing " + newsgroups[newsgroup] + " with "
          //      + msgs.length + " messages");

        // for (int i = 0; i < msgs.length && i < MSGS_PER_GROUP; i++) {
        for (int i = 0; i < msgs.length; i++) {
            numDocs++;
            //System.out.println(msgs[i].getName() + " is "
              //      + newsgroups[newsgroup] + " #" + (i + 1));
            parseMsg(msgs[i], newsgroup);
        }
    }

    // each message has some headers, then a blank line, then the body
    // all we care about is the subject and the body

    public static final String subjectTag = "Subject: ";

    public static void parseMsg(File msg, int newsgroup)
            throws FileNotFoundException, IOException {
        MessageFeatures mf = msg2Feature(msg,newsgroup);
        mfs.add(mf);
    }

    public static final Stemmer stemmer = new Stemmer();

    public static String doStemming(String word) {
        char[] chars = word.toCharArray();
        stemmer.add(chars, chars.length);
        stemmer.stem();
        return stemmer.toString();
    }

    public static final int maxTokenLength = 20;

    public static final int minWordLength = 3;

    public static final Pattern numberRE = Pattern.compile("[0-9]+");

    public static final Pattern wordRE = Pattern.compile("[a-zA-Z'\\-]+");

    public static final Pattern alphanum = Pattern.compile("\\w+");

    public static final Pattern hyperlink = Pattern
            .compile("http\\:\\/\\/(\\w+\\.)+\\w+");

    public static final Pattern email = Pattern
            .compile("[\\w\\-\\.]+@[\\w\\-\\.]+");

    public static String delims = " \t\n\r\f.()\"',-:;/\\?!@";

    public static void parseLine(String line, HashMap map, HashMap allMap) {
        line = line.toLowerCase();

        // first find hyperlinks
        Matcher mLink = hyperlink.matcher(line);
        while (mLink.find())
            recordWord(mLink.group(), map, allMap);

        // now email addresses
        Matcher mEmail = email.matcher(line);
        while (mEmail.find())
            recordWord(mEmail.group(), map, allMap);

        // everything else
        StringTokenizer st = new StringTokenizer(line, delims);
        while (st.hasMoreTokens()) {
            String word = st.nextToken();

            if (word.length() < maxTokenLength) {
                if (numberRE.matcher(word).matches())
                    recordWord(word, map, allMap);
                else {
                    word = doStemming(word);
                    if (wordRE.matcher(word).matches()
                            && word.length() >= minWordLength
                            && !stopWords.contains(word))
                        recordWord(word, map, allMap);
                }
            }
        }
    }

    public static void recordWord(String word, HashMap map, HashMap allMap) {
        // System.out.println(word);
        Integer num = (Integer) map.get(word);
        if (num == null) {
            map.put(word, new Integer(1));
            // recording df, not cf, so only increment df the first time we see
            // it in this doc
            if (allMap != null)
                recordWord(word, allMap, null);
        } else
            map.put(word, new Integer(num.intValue() + 1));
    }

    public static final String[] STRING_ARRAY = new String[0];

    public static void writeMessageFeatures(String filename) throws IOException {
        ObjectOutputStream out = new ObjectOutputStream(new FileOutputStream(
                filename));

        for (int i = 0; i < mfs.size(); i++) {
            System.out.println("Serializing message #" + (i + 1));
            MessageFeatures mf = (MessageFeatures) mfs.get(i);
            out.writeObject(mf);
        }
        out.close();
        System.out.println(mfs.size() + " messages written!");

    }

    public static NumberFormat nf = NumberFormat.getNumberInstance();

    public void myParseMsgs() throws IOException,
            FileNotFoundException {
        
        loadStopWords();
        nf.setMaximumFractionDigits(15);

        // read in messages
        for (int i = 0; i < newsgroups.length; i++)
            parseNewsgroup(i);

        
        //Alternatively, your code could work directly with the corpus instead
        //of using the intermediate MessageFeature if you prefer to do so
        //writeMessageFeatures(args[0]);

    }

    

    public static String[] getVocabulary(HashMap map) throws IOException {
        String[] words = (String[]) map.keySet().toArray(STRING_ARRAY);
        return words;
    }

    public static int[] getVocabularyCount(HashMap map) {
        String[] words = (String[]) map.keySet().toArray(STRING_ARRAY);
        int n = words.length;
        int[] wordsCount = new int[n];
        for (int i = 0; i < words.length; i++) {
            int freq = ((Integer) map.get(words[i])).intValue();
            wordsCount[i] = freq;
        }
        return wordsCount;
    }

    //This library might be useful to convert your test newgroup document into MessageFeatures
    // You could use the newsgroup labelling to later compare it with your classifier output
    // and calculate the accuracy
    public static MessageFeatures msg2Feature(File msg, int newsgroup)
            throws FileNotFoundException, IOException {
        MessageFeatures mf = new MessageFeatures(newsgroup, msg.getName());
        BufferedReader in = new BufferedReader(new FileReader(msg));
        String line;

        // extract headers first
        while ((line = in.readLine()) != null) {
            if (line.startsWith(subjectTag))
                parseLine(line.substring(subjectTag.length()), mf.subject,
                        allSubject);
            else if (line.equals(""))
                break;
        }

        // now the body
        while ((line = in.readLine()) != null)
            parseLine(line, mf.body, allBody);
        
        return mf;
    }

}
