import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Collections;
import java.util.HashMap;
import java.util.Vector;


 class CHIVariable implements Comparable {
    String word;
    double chiValue;
    public CHIVariable(){
        word = null;
        chiValue = 0.0;
    }
    public CHIVariable(String s, double d){
        word = s;
        chiValue = d;
    }
    
    public int compareTo(Object obj){
        CHIVariable c = (CHIVariable)obj;
        if (chiValue > c.chiValue) return -1;
        else if (chiValue == c.chiValue)return 0;
        else return +1; 
    }
}

public class binomialNBCwCHI {
    
    public static final int newsGroupSize = 20;
    
    // Hash Map for each newsgroup (20). This map contains word (key) and number of documents in which this word has been appeared (keyVal)
    public static HashMap[] ng = new HashMap[newsGroupSize];
    
    // Hash Map after feature selection
    public static HashMap[] ngCHI = new HashMap[newsGroupSize];
    
    // Array for keeping number of documents used for training
    public static Integer[] trainingDocs = new Integer[20];
    
    //HashMap for vocabulary
    public static HashMap vocab; 
    
    public static double meanErrorPercent = 0.0;
    
    public static HashMap vocabCHI;
    
    public static Integer[] trainingDocsCHI = new Integer[20];
    
      

    public static void trainBinomialNBC(int k, MyMessageParser mp) throws FileNotFoundException, IOException
    {
        // Initialize HashMap for each news group and trainingDocs array
        for (int i=0; i<newsGroupSize; i++){
            ng[i] = new HashMap();
            trainingDocs[i]=0;
        }
        vocab = new HashMap();
        
        int docN = 0;
        int docNl = 0;
        int len = 0;
        int t1 = 0;
        int prevUpdate = 0;
        
        for (int i = 0; i < newsGroupSize; i++) {
        
            len = mp.counterNewsgroup(i);
            t1 = len/10; 
        
            //lower part
            int maxL = docNl+t1*k;
            int minL = docNl;
            for (int j=minL; j < maxL; j++){
                MessageFeatures mf = (MessageFeatures) mp.mfs.get(j);
                int ngrpnumber = mf.newsgroup;
                trainingDocs [ngrpnumber] = trainingDocs [ngrpnumber]+1;
                HashMap subject = mf.subject;
                HashMap body = mf.body;
                String[] words= (String[])subject.keySet().toArray(new String[0]);
                for (String s:words){
                    Integer num = (Integer) ng[ngrpnumber].get(s);
                    if (num == null) {
                        ng[ngrpnumber].put(s, new Integer(1));
                        Integer numVocab = (Integer) vocab.get(s);
                        if (numVocab == null){
                            vocab.put(s,new Integer(1));
                        }
                        prevUpdate = j;
                    }
                    else{
                        if (prevUpdate != j){
                            ng[ngrpnumber].put(s, new Integer(num.intValue() + 1));
                            prevUpdate = j;
                        }
                    }
                    
                }
               
                words = (String[]) body.keySet().toArray(new String[0]);
                for (String s : words) {
                    //System.out.println("Body: Count for "+s +" is "+body.get(s));
                    Integer num = (Integer) ng[ngrpnumber].get(s);
                    if (num == null) {
                        ng[ngrpnumber].put(s, new Integer(1));
                        Integer numVocab = (Integer) vocab.get(s);
                        if (numVocab == null){
                            vocab.put(s,new Integer(1));
                        }
                        prevUpdate = j;
                    }
                   else{
                       if (prevUpdate != j){
                           ng[ngrpnumber].put(s, new Integer(num.intValue() + 1));
                           prevUpdate = j;
                       }
                   }
                    
                }
               
               
            } //loop for all j ends
            docNl = docNl+len;
        
        
            //upper half
            int maxU = docN+len;
            int minU= docN+(k+1)*t1;
            docN = docN+(k+1)*t1;
            for (int j=minU; j < maxU; j++){
                docN++;
                MessageFeatures mf = (MessageFeatures) mp.mfs.get(j);
                int ngrpnumber = mf.newsgroup;
                trainingDocs [ngrpnumber] = trainingDocs [ngrpnumber]+1;
                HashMap subject = mf.subject;
                HashMap body = mf.body;
                String[] words= (String[])subject.keySet().toArray(new String[0]);
                for (String s:words){
                    //System.out.println(" Subject: Count for "+s +" is "+subject.get(s));
                    Integer num = (Integer) ng[ngrpnumber].get(s);
                    if (num == null) {
                        ng[ngrpnumber].put(s, new Integer(1));
                        Integer numVocab = (Integer) vocab.get(s);
                        if (numVocab == null){
                            vocab.put(s,new Integer(1));
                        }
                        prevUpdate = j;
                                          
                    }
                    else{
                        if (prevUpdate != j){
                            ng[ngrpnumber].put(s, new Integer(num.intValue() + 1));
                            prevUpdate = j;
                        }
                    }
                    
                }//} //loop for all values of k ends
                words = (String[]) body.keySet().toArray(new String[0]);
                for (String s : words) {
                    //System.out.println("Body: Count for "+s +" is "+body.get(s));
                    Integer num = (Integer) ng[ngrpnumber].get(s);
                    if (num == null) {
                        ng[ngrpnumber].put(s, new Integer(1));
                        Integer numVocab = (Integer) vocab.get(s);
                        if (numVocab == null){
                            vocab.put(s,new Integer(1));
                        }
                        prevUpdate = j;
                    }
                   else{
                       if (prevUpdate != j){
                           ng[ngrpnumber].put(s, new Integer(num.intValue() + 1));
                           prevUpdate = j;
                       }
                   }
                    
                }
               
                
            } //loop for all j ends
            
        
            
        } //loop for all newsgroup ends   
        /*System.out.println("Size of the Vocabulary: "+vocab.size());
        for (int i=0; i<20; ++i){
            System.out.println("Size of class "+i+" is "+ng[i].size());
        }*/
    } //Function ends
    
    public static void featureSelection() {
        for (int i=0; i<newsGroupSize; ++i){
            ngCHI[i] = new HashMap();
            trainingDocsCHI[i] = 0;
        }
        vocabCHI = new HashMap();
        
        int N = (18828 - 1876);   
        for (int i=0; i<newsGroupSize; ++i){
        //for (int i=0; i<newsGroupSize; ++i){
           Vector valOfCHI = new Vector();
           String[] words= (String[])vocab.keySet().toArray(new String[0]);
           for (String s:words){
               double A=0.0, B=0.0, C=0.0, D=0.0;
               
               // Compute A ( term and class co-occur)
               Integer num1 = (Integer) ng[i].get(s);
               if (num1 != null){

                             A = num1.intValue();
               }
               else{
                   A = 0;
               }
               
               // Compute C (term occur but class does not occur)
               for (int j=0; j<newsGroupSize; j++){
                   if (i==j) continue;
                   Integer num2 = (Integer) ng[j].get(s);
                   if ( num2 != null){
                       C = C + num2.intValue();
                   }             
               } //loop for j ends
               
               //Compute B (class occur but term does not occur)
               B = trainingDocs[i]-A;
               
               //Compute D (Neither class occur nor term occur)
               for (int j=0; j<newsGroupSize; j++){
                   if (i==j) continue;
                   Integer num2 = (Integer) ng[j].get(s);
                   if ( num2 != null){
                       D = D + (trainingDocs[j] - num2.intValue());
                   } 
                   else {
                       D = D + trainingDocs[j];
                   }
               } //loop for j ends
               
               double temp =  N * (Math.pow( ( (A * D)- (B * C) ),2))/ ( (A+B) * (A+C) * (B+D) * (C+D) );
               /*if (temp > 400){
                   System.out.println("temp: "+temp+" String: "+s);
               }*/
               CHIVariable c = new CHIVariable(s,temp);
               valOfCHI.add(c);
                     
           }//loop for s ends
           
           Collections.sort(valOfCHI);
           for ( int j=0; j<200; ++j){
               CHIVariable c  = (CHIVariable) valOfCHI.get(j);
               Integer num = (Integer) ng[i].get(c.word);
               if (num == null){
                   ngCHI[i].put(c.word, new Integer(0));
                   Integer num1 = (Integer) vocabCHI.get(c.word);
                   if (num1 == null){
                       vocabCHI.put(c.word, new Integer(1));
                   }
               }
               else {
                   ngCHI[i].put(c.word, (Integer)num.intValue());
                   
                   Integer num1 = (Integer) vocabCHI.get(c.word);
                   trainingDocsCHI[i] = trainingDocsCHI[i]+ num.intValue();
                   //System.out.println("Num Value:"+num1);
                   if (num1 == null){
                       vocabCHI.put(c.word, new Integer(1));
                       
                   }
               }
              //System.out.println ("Class "+i+ " Word: "+c.word+" Chi: "+c.chiValue+ " prob: "+num+ " Iteration: "+j);
           }
           //System.out.println("Size of voacabCHI "+vocabCHI.size());
           //System.out.println("Size of trainCHI "+trainingDocsCHI[i]);
           
        }//loop for newsgroup ends
        
        
    }//function feature selection ends
    
    public static void classifyBinomialNBC (int k, MyMessageParser mp) throws FileNotFoundException, IOException{
        int docN1=0;
        int error =0; 
        int totalDoc = 0;
        int len = 0;
        int t1 = 0;
        Double[] probArray = new Double[20];
        for (int i=0; i<newsGroupSize; i++){
            probArray[i]=0.0;
        }
        
        for (int i=0; i<newsGroupSize; i++){
            len = mp.counterNewsgroup(i);
            t1 = len/10;
            totalDoc = totalDoc+t1;
            int min=k*t1+docN1;
            int max = min+t1;
            for (int j=min; j < max; j++){
                MessageFeatures mf = (MessageFeatures) mp.mfs.get(j);
                HashMap subject = mf.subject;
                HashMap body = mf.body;
                
                for (int c=0; c<20; c++){
                    
                    double Prod =0.0;
                    String[] words= (String[])subject.keySet().toArray(new String[0]);
                    for (String s:words){
                        Integer num = (Integer) ngCHI[c].get(s);
                        double prob = 0.0;
                        if (num==null){
                            if ( ((Integer)vocabCHI.get(s)) == null){
                                continue; //ignore a new word
                            }
                            else {   
                                prob=(1.0)/((double)10000);
                                //prob = (1.0)/( (double)vocab.size());
                            }
                            
                        }
                        else{
                            prob=((double)num.intValue())/((double)trainingDocsCHI[c]);
                            //prob = ((double)num.intValue()+1.0)/( ((double)trainingDocs[c])+((double)vocab.size()) );
                        }
                        
                        double logProb = Math.log10(prob);
                        Prod=Prod+logProb;
                    }
                   
                          
                    words = (String[]) body.keySet().toArray(new String[0]);
                    for (String s:words){
                        Integer num = (Integer) ngCHI[c].get(s);
                        double prob=0.0;
                        if (num==null){
                            if ( ((Integer)vocabCHI.get(s)) == null){
                                continue; //ignore a new word
                            }
                            else {   
                                prob=(1.0)/((double)10000);
                                //prob = (1.0)/( (double)vocab.size());
                            }
                        }
                        else{
                           prob=((double)num.intValue())/((double)trainingDocsCHI[c]);
                            //prob = ((double)num.intValue()+1.0)/( ((double)trainingDocs[c])+((double)vocab.size()) ); 
                        }
                        
                        double logProb = Math.log10(prob);
                        Prod=Prod+logProb;
                    }
                    
                    double total = Math.log10((double)(1.0/20.0))+Prod;
                    probArray[c]=total;
                } //loop for class ends
                
                int max1 = 0;
                for (int i1=1;i1<20; ++i1){
                    if (probArray[i1]>probArray[max1]){
                        max1=i1;
                    }
                                  
                } //loop for max ends
                
                if (max1!=mf.newsgroup) error=error+1;
                
                
            }//loop for j ends
            
            docN1=docN1+len;    
                     
        }//loop for newsgroup ends
        double errorPercent = ((double)(error)/(double)(totalDoc));
        meanErrorPercent = meanErrorPercent+errorPercent;
        
        System.out.println (" K: "+k+ " Miss: "+error+ " Hits: "+(totalDoc-error)+" Total Test Doc: "+totalDoc+ " Error#: "+errorPercent);
                      
    }//Function Ends
    
    public static void main(String[] args) throws FileNotFoundException, IOException, ClassNotFoundException {
        
        MyMessageParser mp = new MyMessageParser();
        mp.myParseMsgs();
        for (int k=0; k<10; ++k){
            //System.out.println("K: "+k);
            trainBinomialNBC(k,mp);
            featureSelection();
           classifyBinomialNBC(k,mp);
        }
        System.out.println("Mean Error: "+(meanErrorPercent/10.0));
        System.out.println("Accuracy: "+(1.0-(meanErrorPercent/10.0)));
          
          
    } //main ends

} //class ends
