import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.HashMap;


public class multinomialNBC {
    
    public static final int newsGroupSize = 20;
    
    // Hash Map for each newsgroup (20). This map contains word (key) and number of documents in which this word has been appeared (keyVal)
    public static HashMap[] ng = new HashMap[newsGroupSize];
    
    // Array for keeping number of documents used for training
    public static Integer[] trainingDocs = new Integer[20];
    
    
    

    public static void trainMultinomialNBC(int k, MyMessageParser mp) throws FileNotFoundException, IOException
    {
        // Initialize HashMap for each news group and trainingDocs array
        for (int i=0; i<newsGroupSize; i++){
            ng[i] = new HashMap();
            trainingDocs[i]=0;
        }
        
        int docN = 0;
        int docNl = 0;
        int len = 0;
        int t1 = 0;
        
        for (int i = 0; i < newsGroupSize; i++) {
        
            len = mp.counterNewsgroup(i);
            t1 = len/10;
        
            //lower part
            int maxL = docNl+t1*k;
            int minL = docNl;
            for (int j=minL; j < maxL; j++){
                MessageFeatures mf = (MessageFeatures) mp.mfs.get(j);
                int ngrpnumber = mf.newsgroup;
                //trainingDocs [ngrpnumber] = trainingDocs [ngrpnumber]+1;
                HashMap subject = mf.subject;
                HashMap body = mf.body;
                String[] words= (String[])subject.keySet().toArray(new String[0]);
                for (String s:words){
                    Integer num = (Integer) ng[ngrpnumber].get(s);
                    if (num == null) {
                        ng[ngrpnumber].put(s, new Integer(1));
                    }
                    else{
                       ng[ngrpnumber].put(s, new Integer(num.intValue() + 1));
                    }
                    
                }
               
                words = (String[]) body.keySet().toArray(new String[0]);
                for (String s : words) {
                    //System.out.println("Body: Count for "+s +" is "+body.get(s));
                    Integer num = (Integer) ng[ngrpnumber].get(s);
                    if (num == null) {
                        ng[ngrpnumber].put(s, new Integer(1));
                    }
                   else{
                       ng[ngrpnumber].put(s, new Integer(num.intValue() + 1));
                   }
                    
                }
               
               
            } //loop for all j ends
            docNl = docNl+len;
        
        
            //upper half
            int maxU = docN+len;
            int minU= docN+(k+1)*t1;
            docN = docN+(k+1)*t1;
            for (int j=minU; j < maxU; j++){
                docN++;
                MessageFeatures mf = (MessageFeatures) mp.mfs.get(j);
                int ngrpnumber = mf.newsgroup;
                trainingDocs [ngrpnumber] = trainingDocs [ngrpnumber]+1;
                HashMap subject = mf.subject;
                HashMap body = mf.body;
                String[] words= (String[])subject.keySet().toArray(new String[0]);
                for (String s:words){
                    //System.out.println(" Subject: Count for "+s +" is "+subject.get(s));
                    Integer num = (Integer) ng[ngrpnumber].get(s);
                    if (num == null) {
                        ng[ngrpnumber].put(s, new Integer(1));
                                          
                    }
                    else{
                       ng[ngrpnumber].put(s, new Integer(num.intValue() + 1));
                    }
                    
                }//} //loop for all values of k ends
                words = (String[]) body.keySet().toArray(new String[0]);
                for (String s : words) {
                    //System.out.println("Body: Count for "+s +" is "+body.get(s));
                    Integer num = (Integer) ng[ngrpnumber].get(s);
                    if (num == null) {
                        ng[ngrpnumber].put(s, new Integer(1));
                    }
                   else{
                       ng[ngrpnumber].put(s, new Integer(num.intValue() + 1));
                   }
                    
                }
               
                
            } //loop for all j ends
            
        
            
        } //loop for all newsgroup ends   
    } //Function ends
    
    public static void classifyMultinomialNBC (int k, MyMessageParser mp) throws FileNotFoundException, IOException{
        int docN1=0;
        int error =0; 
        int totalDoc = 0;
        int len = 0;
        int t1 = 0;
        Double[] probArray = new Double[20];
        for (int i=0; i<newsGroupSize; i++){
            probArray[i]=0.0;
        }
        
        for (int i=0; i<newsGroupSize; i++){
            len = mp.counterNewsgroup(i);
            t1 = len/10;
            totalDoc = totalDoc+t1;
            int min=k*t1+docN1;
            int max = min+t1;
            for (int j=min; j < max; j++){
                MessageFeatures mf = (MessageFeatures) mp.mfs.get(j);
                HashMap subject = mf.subject;
                HashMap body = mf.body;
                
                for (int c=0; c<20; c++){
                    
                    double Prod =0.0;
                    String[] words= (String[])subject.keySet().toArray(new String[0]);
                    for (String s:words){
                        Integer num = (Integer) ng[c].get(s);
                        double prob = 0.0;
                        if (num==null){
                            prob=(1.0)/((double)10000);
                        }
                        else{
                            prob=((double)num.intValue())/((double)ng[c].size()); 
                        }
                        
                        double logProb = Math.log10(prob);
                        Prod=Prod+logProb;
                    }
                   
                          
                    words = (String[]) body.keySet().toArray(new String[0]);
                    for (String s:words){
                        Integer num = (Integer) ng[c].get(s);
                        double prob=0.0;
                        if (num==null){
                            prob=(1.0)/((double)10000);
                        }
                        else{
                            prob=((double)num.intValue())/((double)ng[c].size()); 
                        }
                        
                        double logProb = Math.log10(prob);
                        Prod=Prod+logProb;
                    }
                    
                    double total = Math.log10((double)(1.0/20.0))+Prod;
                    probArray[c]=total;
                } //loop for class ends
                
                int max1 = 0;
                for (int i1=1;i1<20; ++i1){
                    if (probArray[i1]>probArray[max1]){
                        max1=i1;
                    }
                                  
                } //loop for max ends
                
                if (max1!=mf.newsgroup) error=error+1;
                
                
            }//loop for j ends
            
            docN1=docN1+len;    
                     
        }//loop for newsgroup ends
        System.out.println (" K :: "+k+ " Error:: "+error+ " out of "+totalDoc+ " test documents \n");
                      
    }//Function Ends
    
    public static void main(String[] args) throws FileNotFoundException, IOException, ClassNotFoundException {
        
        MyMessageParser mp = new MyMessageParser();
        mp.myParseMsgs();
        for (int k=0; k<10; ++k){
            trainMultinomialNBC(k,mp);
            classifyMultinomialNBC(k,mp);
        }
            
          
    } //main ends

} //class ends
